#! /usr/bin/perl

=pod

=head1 NAME

B<prepare_for_cd> - a GPW author kit utility to make PPCD files from a contribution

=head1 VERSION

This manual describes version 0.05.

=head1 SYNOPSIS

Call

 prepare_for_cd article_<AUTHORMNEMONIC>.pod

in the main directory of your German Perl Workshop author kit.
Usually this is done automatically by the provided makefile.

=head1 DESCRIPTION



=head1 AUTHORS

Steffen Schwigon, Jochen Stenzel, 2004.

=cut

# declare version
$VERSION=0.05;


# pragmata
use strict;
use warnings;
use locale;

# load modules
use Pod::Usage;
use File::Copy;
use File::NCopy;
use File::Path;
use Unicode::String qw(latin1 utf8);


# check call
pod2usage unless @ARGV == 1 and $ARGV[0] =~ /^article_(\w+).pod$/;
my $filename = $ARGV[0];

# store author shortcut
my $authorMnemonic = $1;
my $goodies_subdir = "goodies_$authorMnemonic";
my $article_subdir = $authorMnemonic;
my $indexlist_file = "indexlist_$authorMnemonic";

# a few declarations
my @authors;
my @converted_article_lines = ();
my @abstract_lines          = ();
my %bio                     = ();
my $re_title 		    = '^=head1\s+(.*)$';
my $re_head 		    = '^=head(\d+)(.*)$';
my $re_head_bio             = '^=head2\s+bio\s+(.*)$';
my $re_head_author 	    = '^=head2\s+auth?or(s|en)?';
my $re_head_abstract 	    = '^=head2\s+abstra[ck]t';

# slurp in article file
my @article_lines           = <>;

# collect title data
my @title = map {
                 m/$re_title/;
                 $1;
                } grep /$re_title/, @article_lines;

my $title = $title[0];

my $in_bio_section      = 0;
my $in_author_section   = 0;
my $in_abstract_section = 0;
my $cur_bio_name        = '';

# investigate all lines of the original article file
foreach (@article_lines) {
  next if m/$re_title/; # no title

  # enter/leave AUTHOR/ABSTRACT sections
  if (m/$re_head/i and $in_author_section) {
    $in_author_section = 0;
  }
  if (m/$re_head/i and $in_abstract_section) {
    $in_abstract_section = 0;
  }
  if (m/$re_head/i and $in_bio_section) {
    $in_bio_section = 0;
  }
  if (m/$re_head_author/i) {
    $in_author_section = 1;
  }
  if (m/$re_head_abstract/i) {
    $in_abstract_section = 1;
  }
  if (m/$re_head_bio/i) {
    $in_bio_section = 1;
    $cur_bio_name = $1;
    $cur_bio_name =~ s/^\s+//;
    $cur_bio_name =~ s/\s+$//;
  }
  # don't use (some) headlines
  if (m/$re_head_author/i) {
    next;
  }
  if (m/$re_head_abstract/i) {
    next;
  }
  if (m/$re_head_bio/i) {
    next;
  }
  if ($in_author_section) {
    # get author name
    s/\w<+.+>+//g;           # cut email addresses, incl. possible pod markup
    s/[()<>,;]//g;           # cut email addresses, incl. possible pod markup
    s/^\s+//;
    s/\s+$//;
    push @authors, $_ if $_;
    next;
  }
  # bio content
  if ($in_bio_section) {
    $bio{$cur_bio_name} .= $_;
    next;
  }
  # abstract content
  if ($in_abstract_section) {
    push @abstract_lines, $_;
    # no next !
  }
  # reduce headline depth
  if (/$re_head/) {
    my $headlevel = $1;
    $headlevel--;
    $_ = "=head$headlevel$2";
  }
  push @converted_article_lines, $_;
}
# get author 
my $author = join (', ', @authors);

# build title and author directory names
my $pp_title  = str2ppcd_filename ($title);
my $pp_author = str2ppcd_filename ($author);

# anything to do?
if ($pp_title and $pp_author) {

  # build directory names
  my $outdir        = "ws_ppcd";
  my $outdir_bios   = "$outdir/bios";
  my $outdir_talk   = "$outdir/talk";
  my $outdir_author = "$outdir_talk/$pp_author";
  my $outdir_title  = "$outdir_author/$pp_title";
  my $talk_file     = "$outdir_title/article.pod";
  my $abstract_file = "$outdir_title/talk.pod";

  # make directories
  mkpath ($outdir_author) unless -d $outdir_author;
  mkpath ($outdir_title)  unless -d $outdir_title;

  # write talk file
  open TALKFILE, "> $talk_file" or die "Cannot open $talk_file";
  print TALKFILE @converted_article_lines;
  close TALKFILE;

  # write abstract file, if necessary
  if (@abstract_lines) {
    open ABSTRACTFILE, "> $abstract_file" or die "Cannot open $abstract_file";
    print ABSTRACTFILE @abstract_lines;
    close ABSTRACTFILE;
  }

  # copy images and goodies as necessary
  my $ncopy = new File::NCopy (recursive => 1);
  $ncopy->copy ($article_subdir, $outdir_title) if -d $article_subdir;
  mkpath ("$outdir_title/data"), $ncopy->copy ("$goodies_subdir/*", "$outdir_title/data") if <$goodies_subdir/*>;

  # copy index list, if any
  File::Copy::copy ($indexlist_file, "$outdir_title/talk-index") if -e $indexlist_file;

  # copy author bio, if any
  foreach my $bio_name (keys %bio) {
    my $pp_bio_name  = str2ppcd_filename ($bio_name);
    my $bio_dir      = "$outdir_bios/$pp_bio_name";
    my $bio_filename = "$bio_dir/author.pod";
    mkpath ($bio_dir) unless -d $bio_dir;
    open BIOFILE, "> $bio_filename" or die "Cannot open $bio_filename";
    print BIOFILE $bio{$bio_name};
    close BIOFILE;
  }
}


# FUNCTIONS #########################################

sub str2ppcd_filename {
  my $str =  shift;
  $str 	=~ s/^\s+//;
  $str 	=~ s/\s+$//;
  $str 	=~ s/\s/_/g;
  my $utf8_str = latin1($str)->utf8;
  return $utf8_str;
}

